﻿using Microsoft.AspNetCore.Authentication;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using System.Text.Encodings.Web;
using System.Threading.Tasks;

namespace TestAPIServer
{
    /// <summary>
    /// 커스텀 인증 핸들러
    /// </summary>
    public class CustomAuthenticationHandler : AuthenticationHandler<AuthenticationSchemeOptions>
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CustomAuthenticationHandler(options, loggerFactory, urlEncoder, systemClock)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="options">옵션</param>
        /// <param name="loggerFactory">로그 기록기 팩토리</param>
        /// <param name="urlEncoder">URL 인코더</param>
        /// <param name="systemClock">시스템 시계</param>
        public CustomAuthenticationHandler
        (
            IOptionsMonitor<AuthenticationSchemeOptions> options,
            ILoggerFactory                               loggerFactory,
            UrlEncoder                                   urlEncoder,
            ISystemClock                                 systemClock
        ) : base(options, loggerFactory, urlEncoder, systemClock)
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 인증 처리하기 (비동기) - HandleAuthenticateAsync()

        /// <summary>
        /// 인증 처리하기 (비동기)
        /// </summary>
        /// <returns>인증 결과</returns>
        protected override Task<AuthenticateResult> HandleAuthenticateAsync()
        {
            return Task.FromResult(AuthenticateResult.Fail("Failed Authentication"));
        }

        #endregion
    }
}