﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;

namespace TestAPIServer.Requirement
{
    /// <summary>
    /// JWT 요청 권한 핸들러
    /// </summary>
    public class JWTRequirementAuthorizationHandler : AuthorizationHandler<JWTRequirement>
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// HTTP 클라이언트
        /// </summary>
        private readonly HttpClient httpClient;

        /// <summary>
        /// HTTP 컨텍스트
        /// </summary>
        private readonly HttpContext httpContext;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - JWTRequirementAuthorizationHandler(factory, accessor)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="factory">HTTP 클라이언트 팩토리</param>
        /// <param name="accessor">HTTP 컨텍스트 접근자</param>
        public JWTRequirementAuthorizationHandler(IHttpClientFactory factory, IHttpContextAccessor accessor)
        {
            this.httpClient  = factory.CreateClient();
            this.httpContext = accessor.HttpContext;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 요청 처리하기 (비동기) - HandleRequirementAsync(context, requirement)

        /// <summary>
        /// 요청 처리하기 (비동기)
        /// </summary>
        /// <param name="context">컨텍스트</param>
        /// <param name="requirement">요청</param>
        /// <returns>태스크</returns>
        protected override async Task HandleRequirementAsync(AuthorizationHandlerContext context, JWTRequirement requirement)
        {
            if(this.httpContext.Request.Headers.TryGetValue("Authorization", out var authorizationHeaderValue))
            {
                string accessToken = authorizationHeaderValue.ToString().Split(' ')[1];

                string url = $"https://localhost:44300/oauth/validate?access_token={accessToken}";

                HttpResponseMessage response = await this.httpClient.GetAsync(url);

                if(response.StatusCode == HttpStatusCode.OK)
                {
                    context.Succeed(requirement);
                }
            }
        }

        #endregion
    }
}